#
######################################################
#
#  SaVi by Lloyd Wood (lloydwood@users.sourceforge.net),
#          Patrick Worfolk (worfolk@alum.mit.edu) and
#          Robert Thurman.
#
#  Copyright (c) 1997 by The Geometry Center.
#  Also Copyright (c) 2006 by Lloyd Wood.
#
#  This file is part of SaVi.  SaVi is free software;
#  you can redistribute it and/or modify it only under
#  the terms given in the file COPYRIGHT which you should
#  have received along with this file.  SaVi may be
#  obtained from:
#  http://savi.sourceforge.net/
#  http://www.geom.uiuc.edu/locate/SaVi
#
######################################################
#
# fisheye.tcl
#
# $Id: fisheye.tcl,v 1.18 2006/09/16 15:08:24 lloydwood Exp $


proc fisheye(build) {} {
    global fisheye_flag fisheye_flag_dismissed Fisheye COLOR

    if {[eval window(raise) fisheye]} return

    set fisheye_name [build_Toplevel fisheye]

    wm protocol $fisheye_name WM_DELETE_WINDOW fisheye(dismiss)

    build_Title $fisheye_name "Fisheye view of the local sky"

    set cmd [build_CmdFrame $fisheye_name cmd]

    build_Menubar $fisheye_name mbar \
	{"Help" \
             {"about SaVi..." "about(build)"} } \
	{"File" \
	     {"Load satellites..." "load(build)"} \
	     {"Save satellites as Tcl script..." "save(build)"} \
	     {} \
	     {"Save fisheye as postscript..." fisheye(save)} \
	     {} \
	     {"Close" "fisheye(dismiss)"} } \
	{"Edit" \
	     {"Simulation constants..." "params(build)"} } \
	{"Views" \
	     {"Satellites list..." "window(raise) main"} \
	     {"Global coverage..." "coverage(build)"} \
	     {} \
	     {"> Step forwards" "main(forw_step)"} \
	     {"< Step back" "main(back_step)"} \
	     {">> Forwards..." "main(forwards)"} \
	     {"<< Reverse..." "main(backwards)"} \
	     {"Freeze views" "main(stop)"} \
	     {} \
	     {"Restart views at time zero" "main(restart)"} } \
	{"Locations" \
	     {"Equator (0,0)" "fisheye(set_location) 0 0 0 0"} \
	     {} \
	     {"Adelaide, Australia" "fisheye(set_location) -34 55 138 36"} \
	     {"Algiers, Algeria" "fisheye(set_location) 36 50 3 0"} \
	     {"Amsterdam, Netherlands" "fisheye(set_location) 52 22 4 53"} \
	     {"Anchorage, Alaska" "fisheye(set_location) 61 13 -149 54"} \
	     {"Ankara, Turkey" "fisheye(set_location) 39 55 32 55"} \
	     {"Athens, Greece" "fisheye(set_location) 37 58 23 43"} \
	     {"Auckland, New Zealand" "fisheye(set_location) -36 52 174 45"} \
	     {"Bangkok, Thailand" "fisheye(set_location) 13 45 100 30"} \
	     {"Beijing, China" "fisheye(set_location) 39 55 116 25"} \
	     {"Berlin, Germany" "fisheye(set_location) 52 30 13 25"} \
	     {"Bombay, India" "fisheye(set_location) 19 0 72 48"} \
	     {"Brussels, Belgium" "fisheye(set_location) 50 52 4 22"} \
	     {"Buenos Aires, Argentina" "fisheye(set_location) -34 35 -58 22"} \
	     {"Copenhagen, Denmark" "fisheye(set_location) 55 40 12 34"} \
	     {"Dublin, Eire" "fisheye(set_location) 53 20 -6 15"} \
	     {"Lima, Peru" "fisheye(set_location) -12 0 -77 2"} \
	     {"Lisbon, Portugal" "fisheye(set_location) 38 44 -9 9"} \
	     {"London, UK" "fisheye(set_location) 51 32 0 -5"} \
	     {"Madrid, Spain" "fisheye(set_location) 40 26 -3 42"} \
	     {"Mecca, Saudi Arabia" "fisheye(set_location) 21 29 39 45"} \
	     {"Melbourne, Australia" "fisheye(set_location) -37 47 144 58"} \
	     {"Moscow, Russia" "fisheye(set_location) 55 45 37 36"} \
	     {"Oslo, Norway" "fisheye(set_location) 59 57 10 42"} 
	     {"Paris, France" "fisheye(set_location) 48 48 2 20"} \
	     {"Quito, Ecuador" "fisheye(set_location) 0 -14 -78 30"} \
	     {"San Francisco, USA" "fisheye(set_location) 37 47 -122 26"} \
	     {"Reykjavik, Iceland" "fisheye(set_location) 64 4 -21 58"} \
	     {"Rio de Janeiro, Brazil" "fisheye(set_location) -22 57 -43 12"} \
	     {"Rome, Italy" "fisheye(set_location) 41 54 12 27"} \
	     {"Singapore" "fisheye(set_location) 1 14 103 55"} \
	     {"Sydney, Australia" "fisheye(set_location) -34 0 151 0"} \
	     {"Toronto, Canada" "fisheye(set_location) 43 40 -79 24"} \
	     {"Tokyo, Japan" "fisheye(set_location) 35 40 139 45"} \
	     {"Vancouver, Canada" "fisheye(set_location) 49 13 -123 6"} \
	     {"Vienna, Austria" "fisheye(set_location) 48 14 16 20"} \
	     {"Washington DC, USA" "fisheye(set_location) 38 53 -77 02"} \
	     {"Wellington, New Zealand" "fisheye(set_location) -41 17 174 47"} }


    set p "$cmd.p"
    canvas $p -bg black -height 300 -width 300

    pack $p -side right
    pack $p -fill both -expand 1

    build_LabelEntryColumns $cmd le1 \
        {label "" {time fisheye_lat_lon}}

    build_Optionslist $cmd olist \
	{"Reverse fisheye view" fisheye_reverse_view_flag}

    set cmdq [build_StdFrame $cmd q]

    build_LabelEntryColumns $cmdq le0 \
	{text " " {"Lat:" "Long:"}} \
	      {ientry "degrees" {Fisheye(lat_degrees) Fisheye(long_degrees)} } \
	{ientry "minutes" {Fisheye(lat_minutes) Fisheye(long_minutes)} }

    pack $cmdq -side bottom -fill x

    pack $cmd -fill both -expand 1


    # an explicit Dismiss Window button halts computation.
    build_DismissButtonbar $fisheye_name dbbar fisheye(dismiss) \
	{"Set new location" fisheye(apply)}
    fisheye(resize)
    fisheye(update)
    set fisheye_flag $fisheye_flag_dismissed
}

proc fisheye(dismiss) {} {
    global fisheye_flag fisheye_flag_dismissed

    set fisheye_flag_dismissed $fisheye_flag
    set fisheye_flag 0
    destroy .fisheye
}

proc fisheye(draw) {} {
    global FS_D COLOR coverage_angle

    # initialize display

    set p ".fisheye.cmd.p"
    if {[eval winfo exists $p] == 0} return

    set adjust 2

    set d [expr $FS_D - $adjust]

    set x_start [expr $adjust]
    set y_end [expr $d - $x_start]
    $p create oval $x_start $x_start $y_end $y_end -fill white -width 1 -tags disk

    # 30 degrees circle
    set x_start [expr $d / 6 + $adjust]
    set y_end [expr $d - $x_start]
    $p create oval $x_start $x_start $y_end $y_end -outline black -width 1 -tags disk

    # 60 degrees circle
    set x_start [expr $d / 3 + $adjust]
    set y_end [expr $d - $x_start]
    $p create oval $x_start $x_start $y_end $y_end -outline black -width 1 -tags disk


    # centre point
    set r [expr $d / 2]
    set r1 [expr $r + 1]
    $p create rectangle $r $r $r1 $r1 -outline black -width 1 -tags disk

    if {$r > 100} {
      # 15 degrees circle
      set x_start [expr $d / 12 + $adjust]
      set y_end [expr $d - $x_start]
      $p create oval $x_start $x_start $y_end $y_end -outline grey -width 1 -tags disk

      # 45 degrees circle
      set x_start [expr $d / 4 + $adjust]
      set y_end [expr $d - $x_start]
      $p create oval $x_start $x_start $y_end $y_end -outline grey -width 1 -tags disk

      # 75 degrees circle
      set x_start [expr $d * 5/12 + $adjust]
      set y_end [expr $d - $x_start]
      $p create oval $x_start $x_start $y_end $y_end -outline grey -width 1 -tags disk
    }
}

proc fisheye(delete) {} {
    set p ".fisheye.cmd.p"

    if [winfo exists $p] {
	$p delete crosses
	$p delete labels
	$p delete mask_elevation
    }
}

proc fisheye(resize) {} {

    # update C with current smallest dimension of window.

    set p ".fisheye.cmd.p"
    set diam [eval winfo width $p]
    set height [eval winfo height $p]
    if {$diam > $height} {
        set diam $height
    }
    upvar #0 FS_D FS_D
    if {$diam != $FS_D} {
	$p delete disk
	set FS_D $diam
	fisheye(draw)
    }
}

proc fisheye(update) {} {

    set flag fisheye_flag
    set fisheye_flag 1
    if {$flag == 0} {
	set fisheye_flag 0
    }
    update
}



proc fisheye(apply) {} {
    global Fisheye
    satellites FISHEYE_SET $Fisheye(lat_degrees) $Fisheye(lat_minutes) \
	$Fisheye(long_degrees) $Fisheye(long_minutes)
    fisheye(update)
}

proc fisheye(set_location) {lat_degrees lat_minutes long_degrees long_minutes} {
    global Fisheye

    set Fisheye(lat_degrees) $lat_degrees
    set Fisheye(lat_minutes) $lat_minutes
    set Fisheye(long_degrees) $long_degrees
    set Fisheye(long_minutes) $long_minutes
    fisheye(apply)
}

proc fisheye(save) {} {
    global SaVe last_filename

    set types {
        {"Postscript files"  {.ps} }
    }

    set filename [tk_getSaveFile -filetypes $types \
	          -title "SaVi: save fisheye as Postscript" ]

    if {"$filename" == ""} return

    set length [string length "$filename"]
    set last3 [string range "$filename" [expr $length - 3] $length]

    if {[string compare $last3 ".ps"] != 0} {
	set filename "$filename.ps"
    }

    # open file
    set f [open "$filename" w]
    if {$f == ""} {
	puts stderr "\nSaVi: couldn't save postscript to $filename"
	return
    }

    set p ".fisheye.cmd.p"
    $p postscript -channel $f

    # close file
    close $f

    puts stderr "\nSaVi: saved fisheye as postscript to $filename"

    main(title) "$filename"
}

